"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const deps_1 = require("./deps");
const pjson = require('../package.json');
const debug = require('debug')('cli-engine-command');
class Command {
    constructor(config) {
        this.__config = { _version: pjson.version };
        this.options = {};
        this.config = deps_1.deps.Config.buildConfig(config);
        const ctor = this.constructor;
        const props = [
            ['flags', 'flags'],
            ['args', 'args'],
            ['variableArgs', 'strict'],
            ['description', 'description'],
            ['hidden', 'hidden'],
            ['usage', 'usage'],
            ['help', 'help'],
            ['aliases', 'aliases'],
        ];
        for (let [from, to] of props) {
            if (ctor[from]) {
                let cli = new deps_1.deps.CLI({ mock: this.config.mock });
                cli.warn(`${from} is defined as a static property on ${this.__config.id}`);
                this.options[to] = ctor[from];
            }
        }
    }
    static async mock(...argv) {
        const cmd = new this({ argv: ['argv0', 'argv1'].concat(argv), mock: true });
        await cmd._run();
        return {
            cmd,
            stdout: cmd.cli.stdout.output,
            stderr: cmd.cli.stderr.output,
        };
    }
    async _run(argv) {
        try {
            this.options.argv = argv;
            debug('initializing %s version: %s', this.__config.id, this.__config._version);
            await this.init();
            debug('run');
            await this.run();
            debug('done');
            await this.cli.done();
        }
        catch (err) {
            this.cli.error(err);
        }
    }
    async init() {
        this.color = deps_1.deps.Color.color;
        const CLI = deps_1.deps.CLI;
        this.cli = new CLI({ debug: this.config.debug, mock: this.config.mock, errlog: this.config.errlog });
        this.http = deps_1.deps.HTTP.defaults({
            headers: {
                'user-agent': `${this.config.name}/${this.config.version} (${this.config.platform}-${this.config
                    .arch}) node-${process.version}`,
            },
        });
        if (!this.config.mock)
            this.cli.handleUnhandleds();
        this.options.argv = this.options.argv || this.config.argv.slice(2);
        debug('argv: %o', this.options.argv);
        const { argv, flags, args } = await deps_1.deps.CLIFlags.parse(this.options);
        this.flags = flags;
        this.argv = argv;
        this.args = args;
    }
    async run() { }
    async done() {
        this.cli.done();
    }
    buildHelp() {
        let help = new deps_1.deps.Help(this.config);
        return help.command(this);
    }
    buildHelpLine() {
        let help = new deps_1.deps.Help(this.config);
        return help.commandLine(this);
    }
}
exports.Command = Command;
