"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const list_1 = require("cli-ux/lib/list");
const cli_flags_1 = require("cli-flags");
const chalk = require("chalk");
function buildUsage(command) {
    if (command.options.usage)
        return command.options.usage.trim();
    let cmd = command.__config.id;
    if (!command.options.args)
        return (cmd || '').trim();
    let args = command.options.args.map(renderArg);
    return `${cmd} ${args.join(' ')}`.trim();
}
function renderArg(arg) {
    let name = arg.name.toUpperCase();
    if (arg.required)
        return `${name}`;
    else
        return `[${name}]`;
}
class Help {
    constructor(config) {
        this.config = config;
    }
    command(cmd) {
        let flags = Object.entries(cmd.options.flags || {})
            .filter(([, f]) => !f.hidden)
            .map(([k, f]) => {
            f.name = k;
            return f;
        });
        let args = (cmd.options.args || []).filter(a => !a.hidden);
        let hasFlags = flags.length ? ` ${chalk.blue('[flags]')}` : '';
        let usage = `${chalk.bold('Usage:')} ${this.config.bin} ${buildUsage(cmd)}${hasFlags}\n`;
        return [
            usage,
            cmd.options.description ? `\n${chalk.bold(cmd.options.description.trim())}\n` : '',
            this.renderAliases(cmd.options.aliases),
            this.renderArgs(args),
            this.renderFlags(flags),
            cmd.options.help ? `\n${cmd.options.help.trim()}\n` : '',
        ].join('');
    }
    commandLine(cmd) {
        return [buildUsage(cmd), cmd.options.description ? chalk.dim(cmd.options.description) : null];
    }
    renderAliases(aliases) {
        if (!aliases || !aliases.length)
            return '';
        let a = aliases.map(a => `  $ ${this.config.bin} ${a}`).join('\n');
        return `\n${chalk.blue('Aliases:')}\n${a}\n`;
    }
    renderArgs(args) {
        if (!args.find(f => !!f.description))
            return '';
        return ('\n' +
            list_1.renderList(args.map(a => {
                return [a.name.toUpperCase(), a.description ? chalk.dim(a.description) : null];
            })) +
            '\n');
    }
    renderFlags(flags) {
        if (!flags.length)
            return '';
        return `\n${chalk.blue('Flags:')}\n` + list_1.renderList(cli_flags_1.flagUsages(flags)) + '\n';
    }
}
exports.Help = Help;
